use crate::{
    database::{Issue, PartialIssue, PartialUser, User},
    EXECONN,
};
use rocket::serde::json::Json;
use rocket::{get, post};
use std::io;

#[get("/api/issues")]
pub async fn all_issues() -> Json<Vec<(Issue, User)>> {
    // Json(EXECONN.read().await.as_ref().unwrap().all_issues().await)

    let issues = EXECONN.read().await.as_ref().unwrap().all_issues().await;

    let mut new: Vec<(Issue, User)> = Vec::with_capacity(issues.len());

    for issue in issues {
        let user = EXECONN
            .read()
            .await
            .as_ref()
            .unwrap()
            .get_user_by_id(issue.author)
            .await
            .unwrap(); // We can safely unwrap because if issue.author is invalid then the DB is FUBAR

        new.push((issue, user));
    }

    Json(new)
}

#[get("/api/issues/<id>")]
pub async fn get_issue(id: i64) -> Option<Json<(Issue, User)>> {
    let issue = EXECONN.read().await.as_ref().unwrap().get_issue(id).await;

    let user_id = issue.as_ref()?.author;

    let user = EXECONN
        .read()
        .await
        .as_ref()
        .unwrap()
        .get_user_by_id(user_id)
        .await;

    match (issue, user) {
        (Some(a), Some(b)) => Some(Json((a, b))),
        _ => None,
    }
}

#[post("/api/add_issue", format = "json", data = "<user_input>")]
pub async fn add_issue(user_input: Json<PartialIssue>) -> Option<String> {
    let data = (*user_input).clone();

    EXECONN
        .read()
        .await
        .as_ref()
        .unwrap()
        .add_issue(data.repo, data.author, data.title, data.desc)
        .await
        .map(|i| i.to_string())
}

#[post("/api/register", format = "json", data = "<user_input>")]
pub async fn register(user_input: Json<PartialUser>) -> io::Result<()> {
    let data = (*user_input).clone();

    EXECONN
        .read()
        .await
        .as_ref()
        .unwrap()
        .create_user(data.username, data.password_hash)
        .await;

    Ok(())
}

#[get("/api/user/by_username/<username>")]
pub async fn get_user_by_username(username: String) -> Option<Json<User>> {
    let x = EXECONN
        .read()
        .await
        .as_ref()?
        .get_user_by_username(username)
        .await
        .unwrap_or_default();

    Some(Json(x))
}

#[get("/api/user/by_id/<id>")]
pub async fn get_user_by_id(id: i32) -> Option<Json<User>> {
    let x = EXECONN
        .read()
        .await
        .as_ref()?
        .get_user_by_id(id)
        .await
        .unwrap_or_default();

    Some(Json(x))
}
